@@ -63,7 +63,24 @@ if not hasattr(json, "dumps"):
     json.load = load
 
 import pkg_resources
-import xdg, xdg.BaseDirectory
+import gettext
+import locale
+
+# Initialize gettext
+try:
+    if hasattr(locale, "bindtextdomain"):
+        locale.bindtextdomain("deluge", pkg_resources.resource_filename("deluge", "i18n"))
+    if hasattr(locale, "textdomain"):
+        locale.textdomain("deluge")
+    gettext.bindtextdomain("deluge", pkg_resources.resource_filename("deluge", "i18n"))
+    gettext.textdomain("deluge")
+    gettext.install("deluge", pkg_resources.resource_filename("deluge", "i18n"))
+except Exception, e:
+    from deluge.log import LOG as log
+    log.error("Unable to initialize gettext/locale!")
+    log.exception(e)
+    import __builtin__
+    __builtin__.__dict__["_"] = lambda x: x
 
 from deluge.error import *
 
@@ -132,6 +149,7 @@ def get_default_config_dir(filename=None):
         else:
             return os.path.join(os.environ.get("APPDATA"), "deluge")
     else:
+        import xdg.BaseDirectory
         if filename:
             return os.path.join(xdg.BaseDirectory.save_config_path("deluge"), filename)
         else:
@@ -210,7 +228,7 @@ def open_url_in_browser(url):
 
     :param url: the url to open
     :type url: string
-    
+
     """
     import webbrowser
     webbrowser.open(url)
@@ -234,12 +252,12 @@ def fsize(fsize_b):
     """
     fsize_kb = fsize_b / 1024.0
     if fsize_kb < 1024:
-        return "%.1f KiB" % fsize_kb
+        return "%.1f %s" % (fsize_kb, _("KiB"))
     fsize_mb = fsize_kb / 1024.0
     if fsize_mb < 1024:
-        return "%.1f MiB" % fsize_mb
+        return "%.1f %s" % (fsize_mb, _("MiB"))
     fsize_gb = fsize_mb / 1024.0
-    return "%.1f GiB" % fsize_gb
+    return "%.1f %s" % (fsize_gb, _("GiB"))
 
 def fpcnt(dec):
     """
@@ -340,21 +358,21 @@ def ftime(seconds):
 
 def fdate(seconds):
     """
-    Formats a date string in the locale's date representation based on the systems timezone
+    Formats a date time string in the locale's date representation based on the systems timezone
 
     :param seconds: time in seconds since the Epoch
     :type seconds: float
-    :returns: a string in the locale's date representation or "" if seconds < 0
+    :returns: a string in the locale's datetime representation or "" if seconds < 0
     :rtype: string
 
     """
     if seconds < 0:
         return ""
-    return time.strftime("%x", time.localtime(seconds))
+    return time.strftime("%x %X", time.localtime(seconds))
 
 def is_url(url):
     """
-    A simple regex test to check if the URL is valid
+    A simple test to check if the URL is valid
 
     :param url: the url to test
     :type url: string
@@ -367,8 +385,7 @@ def is_url(url):
     True
 
     """
-    import re
-    return bool(re.search('^(https?|ftp|udp)://', url))
+    return url.partition('://')[0] in ("http", "https", "ftp", "udp")
 
 def is_magnet(uri):
     """
@@ -389,30 +406,6 @@ def is_magnet(uri):
         return True
     return False
 
-def fetch_url(url):
-    """
-    Downloads a torrent file from a given URL and checks the file's validity
-
-    :param url: the url of the .torrent file to fetch
-    :type url: string
-    :returns: the filepath to the downloaded file
-    :rtype: string
-
-    """
-    import urllib
-    from deluge.log import LOG as log
-    try:
-        filename, headers = urllib.urlretrieve(url)
-    except IOError:
-        log.debug("Network error while trying to fetch torrent from %s", url)
-    else:
-        if filename.endswith(".torrent") or headers["content-type"] ==\
-        "application/x-bittorrent":
-            return filename
-        else:
-            log.debug("URL doesn't appear to be a valid torrent file: %s", url)
-            return None
-
 def create_magnet_uri(infohash, name=None, trackers=[]):
     """
     Creates a magnet uri
@@ -469,13 +462,13 @@ def free_space(path):
     :type path: string
     :returns: the free space at path in bytes
     :rtype: int
-    
+
     :raises InvalidPathError: if the path is not valid
 
     """
     if not os.path.exists(path):
         raise InvalidPathError("%s is not a valid path" % path)
-        
+
     if windows_check():
         import win32file
         sectors, bytes, free, total = map(long, win32file.GetDiskFreeSpace(path))
@@ -514,47 +507,90 @@ def is_ip(ip):
             return True
     except socket.error:
         return False
-        
+
+def path_join(*parts):
+    """
+    An implementation of os.path.join that always uses / for the separator
+    to ensure that the correct paths are produced when working with internal
+    paths on Windows.
+    """
+    path = ''
+    for part in parts:
+        if not part:
+            continue
+        elif part[0] == '/':
+            path = part
+        elif not path:
+            path = part
+        else:
+            path += '/' + part
+    return path
+
+XML_ESCAPES = ( 
+    ('&', '&amp;'),
+    ('<', '&lt;'),
+    ('>', '&gt;'),
+    ('"', '&quot;'),
+    ("'", '&apos;')
+)
+
+def xml_decode(string):
+    """ 
+    Unescape a string that was previously encoded for use within xml.
+    
+    :param string: The string to escape
+    :type string: string
+    :returns: The unescaped version of the string.
+    :rtype: string
+    """
+    for char, escape in XML_ESCAPES:
+        string = string.replace(escape, char)
+    return string
+
+def xml_encode(string):
+    """ 
+    Escape a string for use within an xml element or attribute.
+    
+    :param string: The string to escape
+    :type string: string
+    :returns: An escaped version of the string.
+    :rtype: string
+    """
+    for char, escape in XML_ESCAPES:
+        string = string.replace(char, escape)
+    return string
+
 class VersionSplit(object):
     """
     Used for comparing version numbers.
-    
+
     :param ver: the version
     :type ver: string
-    
+
     """
     def __init__(self, ver):
         ver = ver.lower()
-        vs = ver.split("_") if "_" in ver else ver.split("-")
-        self.version = vs[0]
+        vs = ver.replace("_", "-").split("-")
+        self.version = [int(x) for x in vs[0].split(".")]
         self.suffix = None
+        self.dev = False
         if len(vs) > 1:
-            for s in ("rc", "alpha", "beta", "dev"):
-                if s in vs[1][:len(s)]:
-                    self.suffix = vs[1]
+            if vs[1].startswith(("rc", "alpha", "beta")):
+                self.suffix = vs[1]
+            if vs[-1] == 'dev':
+                self.dev = True
 
     def __cmp__(self, ver):
         """
         The comparison method.
-        
+
         :param ver: the version to compare with
         :type ver: VersionSplit
-        
-        """
-        
-        if self.version > ver.version or (self.suffix and self.suffix[:3] == "dev"):
-            return 1
-        if self.version < ver.version:
-            return -1
 
-        if self.version == ver.version:
-            if self.suffix == ver.suffix:
-                return 0
-            if self.suffix is None:
-                return 1
-            if ver.suffix is None:
-                return -1
-            if self.suffix < ver.suffix:
-                return -1
-            if self.suffix > ver.suffix:
-                return 1
+        """
+
+        # If there is no suffix we use z because we want final
+        # to appear after alpha, beta, and rc alphabetically.
+        v1 = [self.version, self.suffix or 'z', self.dev]
+        v2 = [ver.version, ver.suffix or 'z', ver.dev]
+        return cmp(v1, v2)
