@@ -1,7 +1,7 @@
 #
 # torrent.py
 #
-# Copyright (C) 2007, 2008 Andrew Resch <andrewresch@gmail.com>
+# Copyright (C) 2007-2009 Andrew Resch <andrewresch@gmail.com>
 #
 # Deluge is free software.
 #
@@ -32,89 +32,48 @@
 #    statement from all source files in the program, then also delete it here.
 #
 
-#
-
-
 """Internal Torrent class"""
 
 import os
 import time
+from urllib import unquote
 from urlparse import urlparse
 
-try:
-    import deluge.libtorrent as lt
-except ImportError:
-    import libtorrent as lt
-    if not (lt.version_major == 0 and lt.version_minor == 14):
-        raise ImportError("This version of Deluge requires libtorrent 0.14!")
+from deluge._libtorrent import lt
 
 import deluge.common
 import deluge.component as component
-from deluge.configmanager import ConfigManager
+from deluge.configmanager import ConfigManager, get_config_dir
 from deluge.log import LOG as log
-
-import deluge.xmlrpclib
+from deluge.event import *
 
 TORRENT_STATE = deluge.common.TORRENT_STATE
 
 class TorrentOptions(dict):
     def __init__(self):
-        self.config = ConfigManager("core.conf")
-        self.default_keys = {
-            "max_download_speed": "max_download_speed_per_torrent",
-            "max_upload_speed": "max_upload_speed_per_torrent",
-            "max_connections": "max_connections_per_torrent",
-            "max_upload_slots": "max_upload_slots_per_torrent",
-            "prioritize_first_last_pieces": "prioritize_first_last_pieces",
-            "auto_managed": "auto_managed",
-            "move_completed": "move_completed",
-            "move_completed_path": "move_completed_path",
-            "file_priorities": [],
-            "compact_allocation": "compact_allocation",
-            "download_location": "download_location",
-            "add_paused": "add_paused"
-        }
-        super(TorrentOptions, self).__setitem__("stop_at_ratio", False)
-        super(TorrentOptions, self).__setitem__("stop_ratio", 2.0)
-        super(TorrentOptions, self).__setitem__("remove_at_ratio", False)
+        config = ConfigManager("core.conf").config
+        options_conf_map = {
+                            "max_connections": "max_connections_per_torrent",
+                            "max_upload_slots": "max_upload_slots_per_torrent",
+                            "max_upload_speed": "max_upload_speed_per_torrent",
+                            "max_download_speed": "max_download_speed_per_torrent",
+                            "prioritize_first_last_pieces": "prioritize_first_last_pieces",
+                            "compact_allocation": "compact_allocation",
+                            "download_location": "download_location",
+                            "auto_managed": "auto_managed",
+                            "stop_at_ratio": "stop_seed_at_ratio",
+                            "stop_ratio": "stop_seed_ratio",
+                            "remove_at_ratio": "remove_seed_at_ratio",
+                            "move_completed": "move_completed",
+                            "move_completed_path": "move_completed_path",
+                            "add_paused": "add_paused",
+                           }
+        for opt_k, conf_k in options_conf_map.iteritems():
+            self[opt_k] = config[conf_k]
+        self["file_priorities"] = []
+        self["mapped_files"] = {}
 
-    def items(self):
-        i = super(TorrentOptions, self).items()
-        for k in self.default_keys:
-            if k not in super(TorrentOptions, self).keys():
-                i.append((k, self.__getitem__(k)))
-
-        return i
-
-    def keys(self):
-        k = super(TorrentOptions, self).keys()
-        for key in self.default_keys.keys():
-            if key not in k:
-                k.append(key)
-        return k
-
-    def iteritems(self):
-        return self.items().iteritems()
-
-    def has_key(self, key):
-        return super(TorrentOptions, self).has_key(key) or key in self.default_keys
-
-    def __setitem__(self, key, value):
-        super(TorrentOptions, self).__setitem__(key, value)
-
-    def __getitem__(self, key):
-        if super(TorrentOptions, self).has_key(key):
-            return super(TorrentOptions, self).__getitem__(key)
-        elif key in self.default_keys:
-            if self.default_keys[key] and self.default_keys[key] in self.config.config:
-                return self.config[self.default_keys[key]]
-            else:
-                return self.default_keys[key]
-        else:
-            raise KeyError
-
-
-class Torrent:
+class Torrent(object):
     """Torrent holds information about torrents added to the libtorrent session.
     """
     def __init__(self, handle, options, state=None, filename=None, magnet=None):
@@ -122,8 +81,16 @@ class Torrent:
         # Get the core config
         self.config = ConfigManager("core.conf")
 
-        self.signals = component.get("SignalManager")
-
+        self.rpcserver = component.get("RPCServer")
+       
+        # This dict holds previous status dicts returned for this torrent
+        # We use this to return dicts that only contain changes from the previous
+        # {session_id: status_dict, ...}
+        self.prev_status = {}
+        from twisted.internet.task import LoopingCall
+        self.prev_status_cleanup_loop = LoopingCall(self.cleanup_prev_status)
+        self.prev_status_cleanup_loop.start(10)
+         
         # Set the libtorrent handle
         self.handle = handle
         # Set the torrent_id for this torrent
@@ -157,9 +124,6 @@ class Torrent:
         except RuntimeError:
             self.torrent_info = None
 
-        # Files dictionary
-        self.files = self.get_files()
-
         # Default total_uploaded to 0, this may be changed by the state
         self.total_uploaded = 0
 
@@ -180,10 +144,6 @@ class Torrent:
             # Set the filename
             self.filename = state.filename
             self.is_finished = state.is_finished
-            # Set the per-torrent queue options
-            self.options["stop_at_ratio"] = state.stop_at_ratio
-            self.options["stop_ratio"] = state.stop_ratio
-            self.options["remove_at_ratio"] = state.remove_at_ratio
         else:
             # Tracker list
             self.trackers = []
@@ -198,13 +158,7 @@ class Torrent:
                 self.trackers.append(tracker)
 
         # Various torrent options
-        # XXX: Disable resolve_countries if using a 64-bit long and on lt 0.14.2 or lower.
-        # This is a workaround for a bug in lt.
-        import sys
-        if sys.maxint > 0xFFFFFFFF and lt.version < "0.14.3.0":
-            self.handle.resolve_countries(False)
-        else:
-            self.handle.resolve_countries(True)
+        self.handle.resolve_countries(True)
 
         self.set_options(self.options)
 
@@ -217,6 +171,9 @@ class Torrent:
         # The tracker status
         self.tracker_status = ""
 
+        # This gets updated when get_tracker_host is called
+        self.tracker_host = None
+
         if state:
             self.time_added = state.time_added
         else:
@@ -228,14 +185,14 @@ class Torrent:
     def set_options(self, options):
         OPTIONS_FUNCS = {
             # Functions used for setting options
-            "max_download_speed": self.set_max_download_speed,
-            "max_upload_speed": self.set_max_upload_speed,
-            "max_connections": self.handle.set_max_connections,
-            "max_upload_slots": self.handle.set_max_uploads,
-            "prioritize_first_last_pieces": self.set_prioritize_first_last,
             "auto_managed": self.set_auto_managed,
-            "file_priorities": self.set_file_priorities,
             "download_location": self.set_save_path,
+            "file_priorities": self.set_file_priorities,
+            "max_connections": self.handle.set_max_connections,
+            "max_download_speed": self.set_max_download_speed,
+            "max_upload_slots": self.handle.set_max_uploads,
+            "max_upload_speed": self.set_max_upload_speed,
+            "prioritize_first_last_pieces": self.set_prioritize_first_last
         }
         for (key, value) in options.items():
             if OPTIONS_FUNCS.has_key(key):
@@ -298,14 +255,14 @@ class Torrent:
     def set_remove_at_ratio(self, remove_at_ratio):
         self.options["remove_at_ratio"] = remove_at_ratio
 
-    def set_move_on_completed(self, move_completed):
+    def set_move_completed(self, move_completed):
         self.options["move_completed"] = move_completed
 
-    def set_move_on_completed_path(self, move_completed_path):
+    def set_move_completed_path(self, move_completed_path):
         self.options["move_completed_path"] = move_completed_path
 
     def set_file_priorities(self, file_priorities):
-        if len(file_priorities) != len(self.files):
+        if len(file_priorities) != len(self.get_files()):
             log.debug("file_priorities len != num_files")
             self.options["file_priorities"] = self.handle.file_priorities()
             return
@@ -339,38 +296,34 @@ class Torrent:
         if trackers == None:
             trackers = []
             for value in self.handle.trackers():
-                if lt.version_minor < 15:
-                    tracker = {}
-                    tracker["url"] = value.url
-                    tracker["tier"] = value.tier
-                else:
-                    tracker = value
+                tracker = {}
+                tracker["url"] = value.url
+                tracker["tier"] = value.tier
                 trackers.append(tracker)
-
             self.trackers = trackers
+            self.tracker_host = None
             return
 
         log.debug("Setting trackers for %s: %s", self.torrent_id, trackers)
         tracker_list = []
 
-        if lt.version_minor < 15:
-            for tracker in trackers:
-                new_entry = lt.announce_entry(tracker["url"])
-                new_entry.tier = tracker["tier"]
-                tracker_list.append(new_entry)
-            self.handle.replace_trackers(tracker_list)
-        else:
-            self.handle.replace_trackers(trackers)
+        for tracker in trackers:
+            new_entry = lt.announce_entry(tracker["url"])
+            new_entry.tier = tracker["tier"]
+            tracker_list.append(new_entry)
+        self.handle.replace_trackers(tracker_list)
 
         # Print out the trackers
         #for t in self.handle.trackers():
-        #    log.debug("tier: %s tracker: %s", t.tier, t.url)
+        #    log.debug("tier: %s tracker: %s", t["tier"], t["url"])
         # Set the tracker list in the torrent object
         self.trackers = trackers
         if len(trackers) > 0:
             # Force a reannounce if there is at least 1 tracker
             self.force_reannounce()
 
+        self.tracker_host = None
+
     ### End Options methods ###
 
     def set_save_path(self, save_path):
@@ -386,6 +339,10 @@ class Torrent:
         LTSTATE = deluge.common.LT_TORRENT_STATE
         ltstate = int(self.handle.status().state)
 
+        # Set self.state to the ltstate right away just incase we don't hit some
+        # of the logic below
+        self.state = str(ltstate)
+
         log.debug("set_state_based_on_ltstate: %s", deluge.common.LT_TORRENT_STATE[ltstate])
         log.debug("session.is_paused: %s", component.get("Core").session.is_paused())
 
@@ -400,7 +357,10 @@ class Torrent:
             return
 
         if ltstate == LTSTATE["Queued"] or ltstate == LTSTATE["Checking"]:
-            self.state = "Checking"
+            if self.handle.is_paused():
+                self.state = "Paused"
+            else:
+                self.state = "Checking"
             return
         elif ltstate == LTSTATE["Downloading"] or ltstate == LTSTATE["Downloading Metadata"]:
             self.state = "Downloading"
@@ -433,12 +393,11 @@ class Torrent:
         else:
             status = self.status
 
-        if self.is_finished and (self.options["stop_at_ratio"] or self.config["stop_seed_at_ratio"]):
+        if self.is_finished and self.options["stop_at_ratio"]:
             # We're a seed, so calculate the time to the 'stop_share_ratio'
             if not status.upload_payload_rate:
                 return 0
-            stop_ratio = self.config["stop_seed_ratio"] if self.config["stop_seed_at_ratio"] else self.options["stop_ratio"]
-
+            stop_ratio = self.options["stop_ratio"]
             return ((status.all_time_download * stop_ratio) - status.all_time_upload) / status.upload_payload_rate
 
         left = status.total_wanted - status.total_done
@@ -460,9 +419,7 @@ class Torrent:
         else:
             status = self.status
 
-        if status.all_time_download > 0:
-            downloaded = status.all_time_download
-        elif status.total_done > 0:
+        if status.total_done > 0:
             # We use 'total_done' if the downloaded value is 0
             downloaded = status.total_done
         else:
@@ -495,11 +452,7 @@ class Torrent:
     def get_peers(self):
         """Returns a list of peers and various information about them"""
         ret = []
-        try:
-            peers = self.handle.get_peer_info()
-        except IndexError, e:
-            log.error("There was an error getting peer info! This may be a bug in libtorrent.  Please upgrade to libtorrent >= 0.14.3.")
-            return ret
+        peers = self.handle.get_peer_info()
 
         for peer in peers:
             # We do not want to report peers that are half-connected
@@ -519,13 +472,13 @@ class Torrent:
                     country += c
 
             ret.append({
-                "ip": "%s:%s" % (peer.ip[0], peer.ip[1]),
-                "up_speed": peer.up_speed,
-                "down_speed": peer.down_speed,
-                "country": country,
                 "client": client,
+                "country": country,
+                "down_speed": peer.down_speed,
+                "ip": "%s:%s" % (peer.ip[0], peer.ip[1]),
+                "progress": peer.progress,
                 "seed": peer.flags & peer.seed,
-                "progress": peer.progress
+                "up_speed": peer.up_speed,
             })
 
         return ret
@@ -541,7 +494,7 @@ class Torrent:
 
         file_progress = self.handle.file_progress()
         ret = []
-        for i,f in enumerate(self.files):
+        for i,f in enumerate(self.get_files()):
             try:
                 ret.append(float(file_progress[i]) / float(f["size"]))
             except ZeroDivisionError:
@@ -552,6 +505,9 @@ class Torrent:
     def get_tracker_host(self):
         """Returns just the hostname of the currently connected tracker
         if no tracker is connected, it uses the 1st tracker."""
+        if self.tracker_host:
+            return self.tracker_host
+
         if not self.status:
             self.status = self.handle.status()
 
@@ -579,11 +535,25 @@ class Torrent:
                         host = ".".join(parts[-3:])
                     else:
                         host = ".".join(parts[-2:])
+                self.tracker_host = host
                 return host
         return ""
 
-    def get_status(self, keys):
-        """Returns the status of the torrent based on the keys provided"""
+    def get_status(self, keys, diff=False):
+        """
+        Returns the status of the torrent based on the keys provided
+        
+        :param keys: the keys to get the status on
+        :type keys: list of str
+        :param diff: if True, will return a diff of the changes since the last
+        call to get_status based on the session_id
+        :type diff: bool
+        
+        :returns: a dictionary of the status keys and their values
+        :rtype: dict
+        
+        """
+     
         # Create the full dictionary
         self.status = self.handle.status()
         if self.handle.has_metadata():
@@ -599,47 +569,47 @@ class Torrent:
 
         #if you add a key here->add it to core.py STATUS_KEYS too.
         full_status = {
-            "distributed_copies": distributed_copies,
-            "total_done": self.status.total_done,
-            "total_uploaded": self.status.all_time_upload,
+            "active_time": self.status.active_time,
             "all_time_download": self.status.all_time_download,
-            "state": self.state,
-            "paused": self.status.paused,
-            "progress": progress,
-            "next_announce": self.status.next_announce.seconds,
-            "total_payload_download": self.status.total_payload_download,
-            "total_payload_upload": self.status.total_payload_upload,
+            "compact": self.options["compact_allocation"],
+            "distributed_copies": distributed_copies,
             "download_payload_rate": self.status.download_payload_rate,
-            "upload_payload_rate": self.status.upload_payload_rate,
+            "file_priorities": self.options["file_priorities"],
+            "hash": self.torrent_id,
+            "is_auto_managed": self.options["auto_managed"],
+            "is_finished": self.is_finished,
+            "max_connections": self.options["max_connections"],
+            "max_download_speed": self.options["max_download_speed"],
+            "max_upload_slots": self.options["max_upload_slots"],
+            "max_upload_speed": self.options["max_upload_speed"],
+            "message": self.statusmsg,
+            "move_on_completed_path": self.options["move_completed_path"],
+            "move_on_completed": self.options["move_completed"],
+            "next_announce": self.status.next_announce.seconds,
             "num_peers": self.status.num_peers - self.status.num_seeds,
             "num_seeds": self.status.num_seeds,
+            "paused": self.status.paused,
+            "prioritize_first_last": self.options["prioritize_first_last_pieces"],
+            "progress": progress,
+            "remove_at_ratio": self.options["remove_at_ratio"],
+            "save_path": self.options["download_location"],
+            "seeding_time": self.status.seeding_time,
+            "seed_rank": self.status.seed_rank,
+            "state": self.state,
+            "stop_at_ratio": self.options["stop_at_ratio"],
+            "stop_ratio": self.options["stop_ratio"],
+            "time_added": self.time_added,
+            "total_done": self.status.total_done,
+            "total_payload_download": self.status.total_payload_download,
+            "total_payload_upload": self.status.total_payload_upload,
             "total_peers": self.status.num_incomplete,
             "total_seeds":  self.status.num_complete,
+            "total_uploaded": self.status.all_time_upload,
             "total_wanted": self.status.total_wanted,
             "tracker": self.status.current_tracker,
             "trackers": self.trackers,
             "tracker_status": self.tracker_status,
-            "save_path": self.options["download_location"],
-            "files": self.files,
-            "file_priorities": self.options["file_priorities"],
-            "compact": self.options["compact_allocation"],
-            "max_connections": self.options["max_connections"],
-            "max_upload_slots": self.options["max_upload_slots"],
-            "max_upload_speed": self.options["max_upload_speed"],
-            "max_download_speed": self.options["max_download_speed"],
-            "prioritize_first_last": self.options["prioritize_first_last_pieces"],
-            "message": self.statusmsg,
-            "hash": self.torrent_id,
-            "active_time": self.status.active_time,
-            "seeding_time": self.status.seeding_time,
-            "seed_rank": self.status.seed_rank,
-            "is_auto_managed": self.options["auto_managed"],
-            "stop_ratio": self.options["stop_ratio"],
-            "stop_at_ratio": self.options["stop_at_ratio"],
-            "remove_at_ratio": self.options["remove_at_ratio"],
-            "move_on_completed": self.options["move_completed"],
-            "move_on_completed_path": self.options["move_completed_path"],
-            "time_added": self.time_added
+            "upload_payload_rate": self.status.upload_payload_rate
         }
 
         def ti_comment():
@@ -653,12 +623,29 @@ class Torrent:
         def ti_name():
             if self.handle.has_metadata():
                 name = self.torrent_info.file_at(0).path.split("/", 1)[0]
+                if not name:
+                    name = self.torrent_info.name()
                 try:
                     return name.decode("utf8", "ignore")
                 except UnicodeDecodeError:
                     return name
 
+            elif self.magnet:
+                try:
+                    keys = dict([k.split('=') for k in self.magnet.split('?')[-1].split('&')])
+                    name = keys.get('dn')
+                    if not name:
+                        return self.torrent_id
+                    name = unquote(name).replace('+', ' ')
+                    try:
+                        return name.decode("utf8", "ignore")
+                    except UnicodeDecodeError:
+                        return name
+                except:
+                    pass
+
             return self.torrent_id
+
         def ti_priv():
             if self.handle.has_metadata():
                 return self.torrent_info.priv()
@@ -682,19 +669,20 @@ class Torrent:
 
         fns = {
             "comment": ti_comment,
+            "eta": self.get_eta,
+            "file_progress": self.get_file_progress,
+            "files": self.get_files,
+            "is_seed": self.handle.is_seed,
             "name": ti_name,
-            "private": ti_priv,
-            "total_size": ti_total_size,
             "num_files": ti_num_files,
             "num_pieces": ti_num_pieces,
-            "piece_length": ti_piece_length,
-            "eta": self.get_eta,
-            "ratio": self.get_ratio,
-            "file_progress": self.get_file_progress,
-            "queue": self.handle.queue_position,
-            "is_seed": self.handle.is_seed,
             "peers": self.get_peers,
-            "tracker_host": self.get_tracker_host
+            "piece_length": ti_piece_length,
+            "private": ti_priv,
+            "queue": self.handle.queue_position,
+            "ratio": self.get_ratio,
+            "total_size": ti_total_size,
+            "tracker_host": self.get_tracker_host,
         }
 
         # Create the desired status dictionary and return it
@@ -710,6 +698,24 @@ class Torrent:
                     status_dict[key] = full_status[key]
                 elif key in fns:
                     status_dict[key] = fns[key]()
+        
+        session_id = self.rpcserver.get_session_id()
+        if diff:
+            if session_id in self.prev_status:
+                # We have a previous status dict, so lets make a diff
+                status_diff = {}
+                for key, value in status_dict.items():
+                    if key in self.prev_status[session_id]:
+                        if value != self.prev_status[session_id][key]:
+                            status_diff[key] = value
+                    else:
+                        status_diff[key] = value
+                
+                self.prev_status[session_id] = status_dict
+                return status_diff
+
+            self.prev_status[session_id] = status_dict
+            return status_dict
 
         return status_dict
 
@@ -720,13 +726,7 @@ class Torrent:
         self.handle.set_upload_limit(int(self.max_upload_speed * 1024))
         self.handle.set_download_limit(int(self.max_download_speed * 1024))
         self.handle.prioritize_files(self.file_priorities)
-        # XXX: Disable resolve_countries if using a 64-bit long and on lt 0.14.2 or lower.
-        # This is a workaround for a bug in lt.
-        import sys
-        if sys.maxint > 0xFFFFFFFF and lt.version < "0.14.3.0":
-            self.handle.resolve_countries(False)
-        else:
-            self.handle.resolve_countries(True)
+        self.handle.resolve_countries(True)
 
     def pause(self):
         """Pause this torrent"""
@@ -738,7 +738,7 @@ class Torrent:
             # show it as 'Paused'.  We need to emit a torrent_paused signal because
             # the torrent_paused alert from libtorrent will not be generated.
             self.update_state()
-            self.signals.emit("torrent_paused", self.torrent_id)
+            component.get("EventManager").emit(TorrentStateChangedEvent(self.torrent_id, "Paused"))
         else:
             try:
                 self.handle.pause()
@@ -760,14 +760,10 @@ class Torrent:
 
             if self.handle.is_finished():
                 # If the torrent has already reached it's 'stop_seed_ratio' then do not do anything
-                if self.config["stop_seed_at_ratio"] or self.options["stop_at_ratio"]:
-                    if self.options["stop_at_ratio"]:
-                        ratio = self.options["stop_ratio"]
-                    else:
-                        ratio = self.config["stop_seed_ratio"]
-
-                    if self.get_ratio() >= ratio:
-                        self.signals.emit("torrent_resume_at_stop_ratio")
+                if self.options["stop_at_ratio"]:
+                    if self.get_ratio() >= self.options["stop_ratio"]:
+                        #XXX: This should just be returned in the RPC Response, no event
+                        #self.signals.emit_event("torrent_resume_at_stop_ratio")
                         return
 
             if self.options["auto_managed"]:
@@ -792,6 +788,14 @@ class Torrent:
 
     def move_storage(self, dest):
         """Move a torrent's storage location"""
+        if not os.path.exists(dest):
+            try:
+                # Try to make the destination path if it doesn't exist
+                os.makedirs(dest)
+            except IOError, e:
+                log.exception(e)
+                log.error("Could not move storage for torrent %s since %s does not exist and could not create the directory.", self.torrent_id, dest)
+                return False
         try:
             self.handle.move_storage(dest.encode("utf8"))
         except:
@@ -805,46 +809,17 @@ class Torrent:
         self.handle.save_resume_data()
         self.waiting_on_resume_data = True
 
-    def write_resume_data(self, resume_data):
-        """Writes the .fastresume file for the torrent"""
-        resume_data = lt.bencode(resume_data)
-        path = "%s/%s.fastresume" % (
-            self.config["state_location"],
-            self.torrent_id)
-        try:
-            self.delete_fastresume()
-            log.debug("Saving fastresume file: %s", path)
-            fastresume = open(path, "wb")
-            fastresume.write(resume_data)
-            fastresume.flush()
-            os.fsync(fastresume.fileno())
-            fastresume.close()
-        except IOError:
-            log.warning("Error trying to save fastresume file")
-
-        self.waiting_on_resume_data = False
-
-    def delete_fastresume(self):
-        """Deletes the .fastresume file"""
-        path = "%s/%s.fastresume" % (
-            self.config["state_location"],
-            self.torrent_id)
-        log.debug("Deleting fastresume file: %s", path)
-        try:
-            os.remove(path)
-        except Exception, e:
-            log.warning("Unable to delete the fastresume file: %s", e)
-
     def write_torrentfile(self):
         """Writes the torrent file"""
         path = "%s/%s.torrent" % (
-            self.config["state_location"],
+            os.path.join(get_config_dir(), "state"),
             self.torrent_id)
         log.debug("Writing torrent file: %s", path)
         try:
-            ti = self.handle.get_torrent_info()
-            md = lt.bdecode(ti.metadata())
-            log.debug("md: %s", md)
+            self.torrent_info = self.handle.get_torrent_info()
+            # Regenerate the file priorities
+            self.set_file_priorities([])
+            md = lt.bdecode(self.torrent_info.metadata())
             torrent_file = {}
             torrent_file["info"] = md
             open(path, "wb").write(lt.bencode(torrent_file))
@@ -854,7 +829,7 @@ class Torrent:
     def delete_torrentfile(self):
         """Deletes the .torrent file in the state"""
         path = "%s/%s.torrent" % (
-            self.config["state_location"],
+            os.path.join(get_config_dir(), "state"),
             self.torrent_id)
         log.debug("Deleting torrent file: %s", path)
         try:
@@ -896,7 +871,7 @@ class Torrent:
         """Renames files in the torrent. 'filenames' should be a list of
         (index, filename) pairs."""
         for index, filename in filenames:
-            self.handle.rename_file(index, filename)
+            self.handle.rename_file(index, filename.encode("utf-8"))
 
     def rename_folder(self, folder, new_folder):
         """Renames a folder within a torrent.  This basically does a file rename
@@ -914,5 +889,16 @@ class Torrent:
             if f["path"].startswith(folder):
                 # Keep a list of filerenames we're waiting on
                 wait_on_folder[2].append(f["index"])
-                self.handle.rename_file(f["index"], f["path"].replace(folder, new_folder, 1))
+                self.handle.rename_file(f["index"], f["path"].replace(folder, new_folder, 1).encode("utf-8"))
         self.waiting_on_folder_rename.append(wait_on_folder)
+        
+    def cleanup_prev_status(self):
+        """
+        This method gets called to check the validity of the keys in the prev_status
+        dict.  If the key is no longer valid, the dict will be deleted.
+        
+        """
+        for key in self.prev_status.keys():
+            if not self.rpcserver.is_session_valid(key):
+                del self.prev_status[key]
+
