@@ -56,6 +56,7 @@ from deluge.configmanager import ConfigManager, get_config_dir
 from deluge.core.torrent import Torrent
 from deluge.core.torrent import TorrentOptions
 import deluge.core.oldstateupgrader
+from deluge.ui.common import utf8_encoded
 
 from deluge.log import LOG as log
 
@@ -191,6 +192,8 @@ class TorrentManager(component.Component):
             self.on_alert_metadata_received)
         self.alerts.register_handler("file_error_alert",
             self.on_alert_file_error)
+        self.alerts.register_handler("file_completed_alert",
+            self.on_alert_file_completed)
 
     def start(self):
         # Get the pluginmanager reference
@@ -225,7 +228,9 @@ class TorrentManager(component.Component):
         # torrent_id is removed from it in self.on_alert_torrent_paused()
         # before we call self.save_resume_data() here.
         save_resume_data_list = []
-        for key in self.torrents.keys():
+        for key in self.torrents:
+            # Stop the status cleanup LoopingCall here
+            self.torrents[key].prev_status_cleanup_loop.stop()
             if not self.torrents[key].handle.is_paused():
                 # We set auto_managed false to prevent lt from resuming the torrent
                 self.torrents[key].handle.auto_managed(False)
@@ -253,16 +258,13 @@ class TorrentManager(component.Component):
 
     def update(self):
         for torrent_id, torrent in self.torrents.items():
-            if self.config["stop_seed_at_ratio"] or torrent.options["stop_at_ratio"] and torrent.state not in ("Checking", "Allocating", "Paused", "Queued"):
+            if torrent.options["stop_at_ratio"] and torrent.state not in ("Checking", "Allocating", "Paused", "Queued"):
                 # If the global setting is set, but the per-torrent isn't.. Just skip to the next torrent
                 # This is so that a user can turn-off the stop at ratio option on a per-torrent basis
-                if self.config["stop_seed_at_ratio"] and not torrent.options["stop_at_ratio"]:
+                if not torrent.options["stop_at_ratio"]:
                     continue
-                stop_ratio = self.config["stop_seed_ratio"]
-                if torrent.options["stop_at_ratio"]:
-                    stop_ratio = torrent.options["stop_ratio"]
-                if torrent.get_ratio() >= stop_ratio and torrent.is_finished:
-                    if self.config["remove_seed_at_ratio"] or torrent.options["remove_at_ratio"]:
+                if torrent.get_ratio() >= torrent.options["stop_ratio"] and torrent.is_finished:
+                    if torrent.options["remove_at_ratio"]:
                         self.remove(torrent_id)
                         break
                     if not torrent.handle.is_paused():
@@ -387,7 +389,7 @@ class TorrentManager(component.Component):
             if options["mapped_files"]:
                 for index, name in options["mapped_files"].items():
                     log.debug("renaming file index %s to %s", index, name)
-                    torrent_info.rename_file(index, name.encode("utf-8"))
+                    torrent_info.rename_file(index, utf8_encoded(name))
 
             add_torrent_params["ti"] = torrent_info
             add_torrent_params["resume_data"] = ""
@@ -401,14 +403,8 @@ class TorrentManager(component.Component):
         else:
             storage_mode = lt.storage_mode_t(1)
 
-        try:
-            # Try to encode this as utf8 if needed
-            options["download_location"] = options["download_location"].encode("utf8")
-        except UnicodeDecodeError:
-            pass
-
         # Fill in the rest of the add_torrent_params dictionary
-        add_torrent_params["save_path"] = options["download_location"]
+        add_torrent_params["save_path"] = utf8_encoded(options["download_location"])
         add_torrent_params["storage_mode"] = storage_mode
         add_torrent_params["paused"] = True
         add_torrent_params["auto_managed"] = False
@@ -479,6 +475,7 @@ class TorrentManager(component.Component):
         # Emit the torrent_added signal
         component.get("EventManager").emit(TorrentAddedEvent(torrent.torrent_id))
 
+        log.info("Torrent %s added by user: %s", torrent.get_status(["name"])["name"], component.get("RPCServer").get_session_user())
         return torrent.torrent_id
 
     def load_torrent(self, torrent_id):
@@ -518,6 +515,8 @@ class TorrentManager(component.Component):
         if torrent_id not in self.torrents:
             raise InvalidTorrentError("torrent_id not in session")
 
+        torrent_name = self.torrents[torrent_id].get_status(["name"])["name"]
+
         # Emit the signal to the clients
         component.get("EventManager").emit(PreTorrentRemovedEvent(torrent_id))
 
@@ -536,6 +535,24 @@ class TorrentManager(component.Component):
         # Remove the .torrent file in the state
         self.torrents[torrent_id].delete_torrentfile()
 
+        # Remove the torrent file from the user specified directory
+        filename = self.torrents[torrent_id].filename
+        if self.config["copy_torrent_file"] \
+            and self.config["del_copy_torrent_file"] \
+            and filename:
+            try:
+                users_torrent_file = os.path.join(
+                    self.config["torrentfiles_location"],
+                    filename)
+                log.info("Delete user's torrent file: %s",
+                    users_torrent_file)
+                os.remove(users_torrent_file)
+            except Exception, e:
+                log.warning("Unable to remove copy torrent file: %s", e)
+
+        # Stop the looping call
+        self.torrents[torrent_id].prev_status_cleanup_loop.stop()
+
         # Remove the torrent from deluge's session
         try:
             del self.torrents[torrent_id]
@@ -547,7 +564,7 @@ class TorrentManager(component.Component):
 
         # Emit the signal to the clients
         component.get("EventManager").emit(TorrentRemovedEvent(torrent_id))
-
+        log.info("Torrent %s removed by user: %s", torrent_name, component.get("RPCServer").get_session_user())
         return True
 
     def load_state(self):
@@ -891,6 +908,7 @@ class TorrentManager(component.Component):
         except:
             return
         torrent.set_save_path(alert.handle.save_path())
+        torrent.set_move_completed(False)
 
     def on_alert_torrent_resumed(self, alert):
         log.debug("on_alert_torrent_resumed")
@@ -996,3 +1014,9 @@ class TorrentManager(component.Component):
         except:
             return
         torrent.update_state()
+
+    def on_alert_file_completed(self, alert):
+        log.debug("file_completed_alert: %s", alert.message())
+        torrent_id = str(alert.handle.info_hash())
+        component.get("EventManager").emit(
+            TorrentFileCompletedEvent(torrent_id, alert.index))
