@@ -32,7 +32,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -42,9 +41,8 @@ import pkg_resources
 
 import deluge.component as component
 import deluge.common
-from deluge.ui.tracker_icons import TrackerIcons
 from deluge.log import LOG as log
-from deluge.ui.client import aclient
+from deluge.ui.client import client
 from deluge.configmanager import ConfigManager
 
 STATE_PIX = {
@@ -76,14 +74,14 @@ def _t(text):
 #sidebar-treeview
 class FilterTreeView(component.Component):
     def __init__(self):
-        component.Component.__init__(self, "FilterTreeView", interval=2000)
+        component.Component.__init__(self, "FilterTreeView", interval=2)
         self.window = component.get("MainWindow")
         glade = self.window.main_glade
         self.hpaned = glade.get_widget("hpaned")
         self.scrolled = glade.get_widget("scrolledwindow_sidebar")
         self.sidebar = component.get("SideBar")
         self.config = ConfigManager("gtkui.conf")
-        self.tracker_icons = TrackerIcons()
+        self.tracker_icons = component.get("TrackerIcons")
 
         self.label_view = gtk.TreeView()
         self.sidebar.add_tab(self.label_view, "filters", _("Filters"))
@@ -197,6 +195,10 @@ class FilterTreeView(component.Component):
             self.select_default_filter()
 
     def update_row(self, cat, value , count):
+        def on_get_icon(icon):
+            if icon:
+                self.set_row_image(cat, value, icon.get_filename())
+
         if (cat, value) in self.filters:
             row = self.filters[(cat, value)]
             self.treestore.set_value(row, 3, count)
@@ -208,8 +210,9 @@ class FilterTreeView(component.Component):
             row = self.treestore.append(self.cat_nodes[cat],[cat, value, label, count , pix, True])
             self.filters[(cat, value)] = row
 
-            if cat == "tracker_host" or cat == "label":
-                self.tracker_icons.get_async(value, lambda filename: self.set_row_image(cat, value, filename))
+            if cat == "tracker_host" and value not in ("All", "Error") and value:
+                d = self.tracker_icons.get(value)
+                d.addCallback(on_get_icon)
 
         self.treestore.set_value(row, FILTER_COLUMN, True)
         return row
@@ -291,7 +294,7 @@ class FilterTreeView(component.Component):
             hide_cat = []
             if not self.config["sidebar_show_trackers"]:
                 hide_cat = ["tracker_host"]
-            aclient.get_filter_tree(self.cb_update_filter_tree, self.config["sidebar_show_zero"], hide_cat)
+            client.core.get_filter_tree(self.config["sidebar_show_zero"], hide_cat).addCallback(self.cb_update_filter_tree)
         except Exception, e:
             log.debug(e)
 
