@@ -32,8 +32,6 @@
 #    statement from all source files in the program, then also delete it here.
 #
 #
-
-
 from deluge.log import LOG as log
 
 # Install the twisted reactor
@@ -82,6 +80,7 @@ from ipcinterface import IPCInterface
 from deluge.ui.tracker_icons import TrackerIcons
 from queuedtorrents import QueuedTorrents
 from addtorrentdialog import AddTorrentDialog
+from deluge.ui.sessionproxy import SessionProxy
 import dialogs
 import common
 
@@ -160,6 +159,7 @@ DEFAULT_PREFS = {
 
 class GtkUI(object):
     def __init__(self, args):
+        self.daemon_bps = (0,0,0)
         # Setup signals
         try:
             import gnome.ui
@@ -167,7 +167,7 @@ class GtkUI(object):
             self.gnome_prog = gnome.init("Deluge", deluge.common.get_version())
             self.gnome_client = gnome.ui.master_client()
             def on_die(*args):
-                gtk.main_quit()
+                reactor.stop()
             self.gnome_client.connect("die", on_die)
             log.debug("GNOME session 'die' handler registered!")
         except Exception, e:
@@ -180,7 +180,7 @@ class GtkUI(object):
             def win_handler(ctrl_type):
                 log.debug("ctrl_type: %s", ctrl_type)
                 if ctrl_type in (CTRL_CLOSE_EVENT, CTRL_SHUTDOWN_EVENT):
-                    gtk.main_quit()
+                    reactor.stop()
                     return 1
             SetConsoleCtrlHandler(win_handler)
 
@@ -208,6 +208,7 @@ class GtkUI(object):
         client.set_disconnect_callback(self.__on_disconnect)
 
         self.trackericons = TrackerIcons()
+        self.sessionproxy = SessionProxy()
         # Initialize various components of the gtkui
         self.mainwindow = MainWindow()
         self.menubar = MenuBar()
@@ -227,6 +228,10 @@ class GtkUI(object):
         # Show the connection manager
         self.connectionmanager = ConnectionManager()
 
+        from twisted.internet.task import LoopingCall
+        rpc_stats = LoopingCall(self.print_rpc_stats)
+        rpc_stats.start(10)
+
         reactor.callWhenRunning(self._on_reactor_start)
         # Start the gtk main loop
         gtk.gdk.threads_enter()
@@ -247,15 +252,28 @@ class GtkUI(object):
         # Shutdown all components
         component.shutdown()
 
-        if self.started_in_classic:
-            try:
-                client.daemon.shutdown()
-            except:
-                pass
-
         # Make sure the config is saved.
         self.config.save()
 
+    def print_rpc_stats(self):
+        import time
+        try:
+            recv = client.get_bytes_recv()
+            sent = client.get_bytes_sent()
+        except AttributeError:
+            return
+
+        log.debug("sent: %s recv: %s", deluge.common.fsize(sent), deluge.common.fsize(recv))
+        t = time.time()
+        delta_time = t - self.daemon_bps[0]
+        delta_sent = sent - self.daemon_bps[1]
+        delta_recv = recv - self.daemon_bps[2]
+
+        sent_rate = deluge.common.fspeed(float(delta_sent) / float(delta_time))
+        recv_rate = deluge.common.fspeed(float(delta_recv) / float(delta_time))
+        log.debug("sent rate: %s recv rate: %s", sent_rate, recv_rate)
+        self.daemon_bps = (t, sent, recv)
+
     def _on_reactor_start(self):
         log.debug("_on_reactor_start")
         self.mainwindow.first_show()
