@@ -31,6 +31,8 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
+#
+
 
 import gtk, gtk.glade
 import os.path
@@ -39,15 +41,15 @@ import pkg_resources
 import gobject
 from itertools import izip
 
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 from deluge.configmanager import ConfigManager
 import deluge.configmanager
 import deluge.component as component
 import deluge.common
-import deluge.ui.gtkui.common as common
 from deluge.ui.gtkui.listview import cell_data_speed as cell_data_speed
 from deluge.ui.gtkui.torrentdetails import Tab
 from deluge.log import LOG as log
+from deluge.ui.countries import COUNTRIES
 
 def cell_data_progress(column, cell, model, row, data):
     value = model.get_value(row, data)
@@ -68,7 +70,9 @@ class PeersTab(Tab):
             })
 
         self.listview = glade.get_widget("peers_listview")
+        self.listview.props.has_tooltip = True
         self.listview.connect("button-press-event", self._on_button_press_event)
+        self.listview.connect("query-tooltip", self._on_query_tooltip)
         # country pixbuf, ip, client, downspeed, upspeed, country code, int_ip, seed/peer icon, progress
         self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, str, int, int, str, float, gtk.gdk.Pixbuf, float)
         self.cached_flag_pixbufs = {}
@@ -253,7 +257,7 @@ class PeersTab(Tab):
             self.peers = {}
             self.torrent_id = torrent_id
 
-        client.get_torrent_status(self._on_get_torrent_status, torrent_id, ["peers"])
+        component.get("SessionProxy").get_torrent_status(torrent_id, ["peers"]).addCallback(self._on_get_torrent_status)
 
     def get_flag_pixbuf(self, country):
         if country == "  ":
@@ -351,8 +355,43 @@ class PeersTab(Tab):
             self.peer_menu.popup(None, None, None, event.button, event.time)
             return True
 
+    def _on_query_tooltip(self, widget, x, y, keyboard_tip, tooltip):
+        if not widget.get_tooltip_context(x, y, keyboard_tip):
+            return False
+        else:
+            model, path, iter = widget.get_tooltip_context(x, y, keyboard_tip)
+
+            country_code = model.get(iter, 5)[0]
+            if country_code != "  " and country_code in COUNTRIES:
+                tooltip.set_text(COUNTRIES[country_code])
+                # widget here is self.listview
+                widget.set_tooltip_cell(tooltip, path, widget.get_column(0),
+                                        None)
+                return True
+            else:
+                return False
+
     def _on_menuitem_add_peer_activate(self, menuitem):
         """This is a callback for manually adding a peer"""
         log.debug("on_menuitem_add_peer")
-        common.add_peer_dialog()
+        dialog_glade = gtk.glade.XML(
+            pkg_resources.resource_filename("deluge.ui.gtkui",
+                "glade/dgtkpopups.glade"))
+        peer_dialog = dialog_glade.get_widget("connect_peer_dialog")
+        txt_ip = dialog_glade.get_widget("txt_ip")
+        response = peer_dialog.run()
+        if response:
+            value = txt_ip.get_text()
+            if ']' in value:
+                #ipv6
+                ip = value.split("]")[0][1:]
+                port = value.split("]")[1][1:]
+            else:
+                #ipv4
+                ip = value.split(":")[0]
+                port = value.split(":")[1]
+            if deluge.common.is_ip(ip):
+                log.debug("adding peer %s to %s", value, self.torrent_id)
+                client.core.connect_peer(self.torrent_id, ip, port)
+        peer_dialog.destroy()
         return True
