@@ -180,6 +180,11 @@ class StatusBar(component.Component):
         self.dht_item = StatusBarItem(
             image=deluge.common.get_pixmap("dht16.png"), tooltip=_("DHT Nodes"))
 
+        self.diskspace_item = self.add_item(
+                stock=gtk.STOCK_HARDDISK,
+                callback=self._on_diskspace_item_clicked,
+                tooltip=_("Free Disk Space"))
+
         self.health_item = self.add_item(
                 stock=gtk.STOCK_DIALOG_ERROR,
                 text=_("No Incoming Connections!"),
@@ -195,7 +200,6 @@ class StatusBar(component.Component):
         client.core.get_config_value(
             "max_upload_speed").addCallback(self._on_max_upload_speed)
         client.core.get_config_value("dht").addCallback(self._on_dht)
-        client.core.get_health().addCallback(self._on_get_health)
 
         self.send_status_request()
 
@@ -209,6 +213,7 @@ class StatusBar(component.Component):
             self.remove_item(self.not_connected_item)
             self.remove_item(self.health_item)
             self.remove_item(self.traffic_item)
+            self.remove_item(self.diskspace_item)
         except Exception, e:
             log.debug("Unable to remove StatusBar item: %s", e)
         self.show_not_connected()
@@ -266,17 +271,20 @@ class StatusBar(component.Component):
     def send_status_request(self):
         # Sends an async request for data from the core
         client.core.get_num_connections().addCallback(self._on_get_num_connections)
-        if self.dht_status:
-            client.core.get_dht_nodes().addCallback(self._on_get_dht_nodes)
-        client.core.get_session_status([
+        keys = [
             "upload_rate",
             "download_rate",
             "payload_upload_rate",
-            "payload_download_rate"]).addCallback(self._on_get_session_status)
+            "payload_download_rate"]
+
+        if self.dht_status:
+            keys.append("dht_nodes")
 
         if not self.health:
-            # Only request health status while False
-            client.core.get_health().addCallback(self._on_get_health)
+            keys.append("has_incoming_connections")
+
+        client.core.get_session_status(keys).addCallback(self._on_get_session_status)
+        client.core.get_free_space().addCallback(self._on_get_free_space)
 
     def on_configvaluechanged_event(self, key, value):
         """
@@ -295,16 +303,12 @@ class StatusBar(component.Component):
         self.num_connections = num_connections
         self.update_connections_label()
 
-    def _on_get_dht_nodes(self, dht_nodes):
-        self.dht_nodes = dht_nodes
-        self.update_dht_label()
-
     def _on_dht(self, value):
         self.dht_status = value
         if value:
             self.hbox.pack_start(
                 self.dht_item.get_eventbox(), expand=False, fill=False)
-            client.core.get_dht_nodes().addCallback(self._on_get_dht_nodes)
+            self.send_status_request()
         else:
             self.remove_item(self.dht_item)
 
@@ -317,6 +321,18 @@ class StatusBar(component.Component):
         self.update_upload_label()
         self.update_traffic_label()
 
+        if "dht_nodes" in status:
+            self.dht_nodes = status["dht_nodes"]
+            self.update_dht_label()
+
+        if "has_incoming_connections" in status:
+            self.health = status["has_incoming_connections"]
+            if self.health:
+                self.remove_item(self.health_item)
+
+    def _on_get_free_space(self, space):
+        self.diskspace_item.set_text(deluge.common.fsize(space))
+
     def _on_max_download_speed(self, max_download_speed):
         self.max_download_speed = max_download_speed
         self.update_download_label()
@@ -325,11 +341,6 @@ class StatusBar(component.Component):
         self.max_upload_speed = max_upload_speed
         self.update_upload_label()
 
-    def _on_get_health(self, value):
-        self.health = value
-        if self.health:
-            self.remove_item(self.health_item)
-
     def update_connections_label(self):
         # Set the max connections label
         if self.max_connections < 0:
@@ -462,3 +473,6 @@ class StatusBar(component.Component):
 
     def _on_traffic_item_clicked(self, widget, event):
         component.get("Preferences").show("Network")
+
+    def _on_diskspace_item_clicked(self, widget, event):
+        component.get("Preferences").show("Downloads")
