@@ -31,7 +31,6 @@
 #    this exception statement from your version. If you delete this exception
 #    statement from all source files in the program, then also delete it here.
 #
-
 #
 
 
@@ -39,15 +38,15 @@ import gtk
 import pkg_resources
 
 import deluge.component as component
-from deluge.ui.client import aclient as client
+from deluge.ui.client import client
 import deluge.common
 from deluge.configmanager import ConfigManager
 from deluge.log import LOG as log
-import deluge.ui.gtkui.common as common
+import common
 
 class SystemTray(component.Component):
     def __init__(self):
-        component.Component.__init__(self, "SystemTray", interval=4000)
+        component.Component.__init__(self, "SystemTray", interval=4)
         self.window = component.get("MainWindow")
         self.config = ConfigManager("gtkui.conf")
         # List of widgets that need to be hidden when not connected to a host
@@ -83,7 +82,7 @@ class SystemTray(component.Component):
             pkg_resources.resource_filename("deluge.ui.gtkui",
                                             "glade/tray_menu.glade"))
 
-        if deluge.common.windows_check():
+        if deluge.common.windows_check() or deluge.common.osx_check():
             self.tray = gtk.status_icon_new_from_pixbuf(
                 common.get_logo(32))
         else:
@@ -118,14 +117,8 @@ class SystemTray(component.Component):
         self.tray_glade.get_widget("upload-limit-image").set_from_file(
             deluge.common.get_pixmap("seeding16.png"))
 
-        if self.config["classic_mode"]:
-            self.hide_widget_list.remove("menuitem_quitdaemon")
-            self.hide_widget_list.remove("separatormenuitem4")
-            self.tray_glade.get_widget("menuitem_quitdaemon").hide()
-            self.tray_glade.get_widget("separatormenuitem4").hide()
-
-        component.get("Signals").connect_to_signal("config_value_changed", self.config_value_changed)
-        if client.get_core_uri() == None:
+        client.register_event_handler("ConfigValueChangedEvent", self.config_value_changed)
+        if not client.connected():
         # Hide menu widgets because we're not connected to a host.
             for widget in self.hide_widget_list:
                 self.tray_glade.get_widget(widget).hide()
@@ -136,6 +129,13 @@ class SystemTray(component.Component):
 
     def __start(self):
         if self.config["enable_system_tray"]:
+
+            if self.config["classic_mode"]:
+                self.hide_widget_list.remove("menuitem_quitdaemon")
+                self.hide_widget_list.remove("separatormenuitem4")
+                self.tray_glade.get_widget("menuitem_quitdaemon").hide()
+                self.tray_glade.get_widget("separatormenuitem4").hide()
+
             # Show widgets in the hide list because we've connected to a host
             for widget in self.hide_widget_list:
                 self.tray_glade.get_widget(widget).show()
@@ -144,10 +144,10 @@ class SystemTray(component.Component):
             self.build_tray_bwsetsubmenu()
 
             # Get some config values
-            client.get_config_value(
-                self._on_max_download_speed, "max_download_speed")
-            client.get_config_value(
-                self._on_max_upload_speed, "max_upload_speed")
+            client.core.get_config_value(
+                "max_download_speed").addCallback(self._on_max_download_speed)
+            client.core.get_config_value(
+                "max_upload_speed").addCallback(self._on_max_upload_speed)
             self.send_status_request()
 
     def start(self):
@@ -162,13 +162,16 @@ class SystemTray(component.Component):
             except Exception, e:
                 log.debug("Unable to hide system tray menu widgets: %s", e)
 
+            self.tray.set_tooltip(_("Deluge\nNot Connected.."))
+
     def shutdown(self):
         if self.config["enable_system_tray"]:
             self.tray.set_visible(False)
 
     def send_status_request(self):
-        client.get_download_rate(self._on_get_download_rate)
-        client.get_upload_rate(self._on_get_upload_rate)
+        client.core.get_session_status([
+            "payload_upload_rate",
+            "payload_download_rate"]).addCallback(self._on_get_session_status)
 
     def config_value_changed(self, key, value):
         """This is called when we received a config_value_changed signal from
@@ -182,18 +185,19 @@ class SystemTray(component.Component):
             self.max_download_speed = max_download_speed
             self.build_tray_bwsetsubmenu()
 
-    def _on_get_download_rate(self, download_rate):
-        self.download_rate = deluge.common.fsize(download_rate)
-
     def _on_max_upload_speed(self, max_upload_speed):
         if self.max_upload_speed != max_upload_speed:
             self.max_upload_speed = max_upload_speed
             self.build_tray_bwsetsubmenu()
 
-    def _on_get_upload_rate(self, upload_rate):
-        self.upload_rate = deluge.common.fsize(upload_rate)
+    def _on_get_session_status(self, status):
+        self.download_rate = deluge.common.fsize(status["payload_download_rate"])
+        self.upload_rate = deluge.common.fsize(status["payload_upload_rate"])
 
     def update(self):
+        if not self.config["enable_system_tray"]:
+            return
+
         # Set the tool tip text
         max_download_speed = self.max_download_speed
         max_upload_speed = self.max_upload_speed
@@ -273,9 +277,9 @@ class SystemTray(component.Component):
             self.window.hide()
         else:
             if self.config["lock_tray"]:
-                if not self.unlock_tray():
-                    return
-            self.window.present()
+                self.unlock_tray()
+            else:
+                self.window.present()
 
     def on_tray_popup(self, status_icon, button, activate_time):
         """Called when the tray icon is right clicked."""
@@ -296,43 +300,40 @@ class SystemTray(component.Component):
         log.debug("on_menuitem_show_deluge_activate")
         if menuitem.get_active() and not self.window.visible():
             if self.config["lock_tray"]:
-                if not self.unlock_tray():
-                    return
-            self.window.present()
+                self.unlock_tray()
+            else:
+                self.window.present()
         elif not menuitem.get_active() and self.window.visible():
             self.window.hide()
 
     def on_menuitem_add_torrent_activate(self, menuitem):
         log.debug("on_menuitem_add_torrent_activate")
-        from addtorrentdialog import AddTorrentDialog
-        client.add_torrent_file(AddTorrentDialog().show())
+        component.get("AddTorrentDialog").show()
 
     def on_menuitem_pause_all_activate(self, menuitem):
         log.debug("on_menuitem_pause_all_activate")
-        client.pause_all_torrents()
+        client.core.pause_all_torrents()
 
     def on_menuitem_resume_all_activate(self, menuitem):
         log.debug("on_menuitem_resume_all_activate")
-        client.resume_all_torrents()
+        client.core.resume_all_torrents()
 
     def on_menuitem_quit_activate(self, menuitem):
         log.debug("on_menuitem_quit_activate")
-        if self.config["lock_tray"]:
-            if not self.unlock_tray():
-                return
+        if self.config["lock_tray"] and not self.window.visible():
+            self.unlock_tray()
 
         if self.config["classic_mode"]:
-            client.shutdown()
+            client.daemon.shutdown()
 
         self.window.quit()
 
     def on_menuitem_quitdaemon_activate(self, menuitem):
         log.debug("on_menuitem_quitdaemon_activate")
-        if self.config["lock_tray"]:
-            if not self.unlock_tray():
-                return
+        if self.config["lock_tray"] and not self.window.visible():
+            self.unlock_tray()
 
-        client.shutdown()
+        client.daemon.shutdown()
         self.window.quit()
 
     def tray_setbwdown(self, widget, data=None):
@@ -355,7 +356,7 @@ class SystemTray(component.Component):
                 return
 
         # Set the config in the core
-        client.set_config({core_key: value})
+        client.core.set_config({core_key: value})
 
         self.build_tray_bwsetsubmenu()
 
@@ -366,7 +367,6 @@ class SystemTray(component.Component):
             from sha import new as sha_hash
 
         log.debug("Show tray lock dialog")
-        result = False
 
         if is_showing_dlg[0]:
             return
@@ -376,25 +376,47 @@ class SystemTray(component.Component):
         entered_pass.set_activates_default(True)
         entered_pass.set_width_chars(25)
         entered_pass.set_visibility(False)
-        entered_pass.show()
-        tray_lock = gtk.Dialog(title=_("Deluge is locked"), parent=None,
-            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK,
-            gtk.RESPONSE_ACCEPT))
-        label = gtk.Label(_("Deluge is password protected.\nTo show the Deluge \
-window, please enter your password"))
-        label.set_line_wrap(True)
-        label.set_justify(gtk.JUSTIFY_CENTER)
-        tray_lock.set_position(gtk.WIN_POS_CENTER_ALWAYS)
-        tray_lock.set_size_request(400, 200)
-        tray_lock.set_default_response(gtk.RESPONSE_ACCEPT)
-        tray_lock.vbox.pack_start(label)
-        tray_lock.vbox.pack_start(entered_pass)
-        tray_lock.show_all()
-        if tray_lock.run() == gtk.RESPONSE_ACCEPT:
-            if self.config["tray_password"] == sha_hash(\
-                entered_pass.get_text()).hexdigest():
-                result = True
-        tray_lock.destroy()
-        is_showing_dlg[0] = False
 
-        return result
+        tray_lock = gtk.Dialog(title="", parent=self.window.window,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK,
+            gtk.RESPONSE_OK))
+        tray_lock.set_default_response(gtk.RESPONSE_OK)
+        tray_lock.set_has_separator(False)
+
+        tray_lock.set_border_width(5)
+
+        hbox = gtk.HBox(spacing=5)
+
+        image = gtk.image_new_from_file(deluge.common.get_pixmap("lock48.png"))
+        image.set_alignment(0.5, 0.0)
+        hbox.pack_start(image, False)
+
+        vbox = gtk.VBox(spacing=5)
+        hbox.pack_start(vbox, False)
+
+        label = gtk.Label(_("<b><big>Deluge is password protected!</big></b>"))
+        label.set_use_markup(True)
+        label.set_alignment(0.0, 0.5)
+        label.set_line_wrap(True)
+        vbox.pack_start(label, False)
+
+        tlabel = gtk.Label(_("<i>Enter your password to continue</i>"))
+        tlabel.set_use_markup(True)
+        tlabel.set_alignment(0.0, 0.5)
+        tlabel.set_line_wrap(True)
+        vbox.pack_start(tlabel, False)
+
+        vbox.pack_start(entered_pass)
+
+        tray_lock.vbox.pack_start(hbox)
+
+        def on_response(dialog, response_id):
+            if response_id == gtk.RESPONSE_OK:
+                if self.config["tray_password"] == sha_hash(entered_pass.get_text()).hexdigest():
+                    self.window.present()
+
+            tray_lock.destroy()
+            is_showing_dlg[0] = False
+
+        tray_lock.connect("response", on_response)
+        tray_lock.show_all()
